@@ -1,9 +1,10 @@
 PythonSum = sum
 PythonAll = all
 import numpy as np
-from numpy import all, any, logical_and, logical_not, isscalar, where, inf, logical_or, logical_xor, isnan
+from numpy import all, any, logical_and, logical_not, isscalar, inf, logical_or, logical_xor, isnan#where
 from operator import gt as Greater, lt as Less, truediv as td
-from FDmisc import update_mul_inf_zero, update_div_zero
+from FDmisc import update_mul_inf_zero, update_div_zero, where
+
 import operator
 
 try:
@@ -52,16 +53,20 @@ class surf(object):
 #    split = lambda self, inds: [extract(self, ind) for ind in inds]
     
     #self.resolve(domain, GREATER)
-    minimum = lambda self, domain, domain_ind = slice(None): \
+    minimum = lambda self, domain, domain_ind = None: \
     self.c +\
+    (PythonSum(where(v > 0, 
+    domain[k][0], domain[k][1])*v for k, v in self.d.items()) if domain_ind is None else
     PythonSum(where(v > 0, 
-    domain[k][0][domain_ind], domain[k][1][domain_ind])*v for k, v in self.d.items())
+    domain[k][0][domain_ind], domain[k][1][domain_ind])*v for k, v in self.d.items()))
     
     #self.resolve(domain, LESS)
-    maximum = lambda self, domain, domain_ind = slice(None): \
+    maximum = lambda self, domain, domain_ind = None: \
     self.c +\
-    PythonSum(where(v  < 0, 
-    domain[k][0][domain_ind], domain[k][1][domain_ind])*v for k, v in self.d.items())
+    (PythonSum(where(v < 0, 
+    domain[k][0], domain[k][1])*v for k, v in self.d.items()) if domain_ind is None else
+    PythonSum(where(v < 0, 
+    domain[k][0][domain_ind], domain[k][1][domain_ind])*v for k, v in self.d.items()))
     
     def render(self, domain, cmp):
         self.rendered = dict((k, where(cmp(v, 0), domain[k][0], domain[k][1])*v) for k, v in self.d.items())
@@ -508,8 +513,13 @@ def boundsurf_abs(b):
 
 
 def Join(inds, arrays):
+#    print(type(inds), type(arrays), len(inds), len(arrays))
+#    print(PythonSum(ind.size for ind in inds), arrays[0].dtype)
     r = np.empty(PythonSum(ind.size for ind in inds), arrays[0].dtype)
+#    print(r.shape, r.dtype)
     for ind, arr in zip(inds, arrays):
+        if ind.size == 0: continue
+#        print (ind.shape, arr.shape)
         r[ind] = arr
     return r
 
@@ -547,7 +557,9 @@ def boundsurf_join(inds, B):
 #    )
 
 def split(*conditions):
-    Rest = np.ones_like(conditions[0]) # dtype bool
+    #Rest = np.ones_like(conditions[0]) # dtype bool
+    #Temporary for PyPy:
+    Rest = np.ones(conditions[0].shape, conditions[0].dtype) 
     r = []
     for c in conditions:
         tmp = logical_and(c, Rest)
